@@ -25,7 +25,7 @@ from .definitions import OM_Measurement, FOOT
 
 projections = {}
 
-IDREGEX = re.compile(r'(?P<id>\(\d+\))')
+IDREGEX = re.compile(r"(?P<id>\(\d+\))")
 
 
 def get_items(start_url):
@@ -34,12 +34,12 @@ def get_items(start_url):
     def rget(url):
         resp = requests.get(url)
         data = resp.json()
-        values = data['value']
-        logging.info('url={}, nvalues={}'.format(url, len(values)))
+        values = data["value"]
+        logging.info("url={}, nvalues={}".format(url, len(values)))
 
         items.extend(values)
         try:
-            next_url = data['@iot.nextLink']
+            next_url = data["@iot.nextLink"]
         except KeyError:
             return
         rget(next_url)
@@ -54,8 +54,8 @@ def make_geometry_point_from_utm(e, n, zone=None, ellps=None, srid=None):
             p = projections[zone]
         else:
             if ellps is None:
-                ellps = 'WGS84'
-            p = pyproj.Proj(proj='utm', zone=int(zone), ellps=ellps)
+                ellps = "WGS84"
+            p = pyproj.Proj(proj="utm", zone=int(zone), ellps=ellps)
             projections[zone] = p
     elif srid:
         # get zone
@@ -64,18 +64,18 @@ def make_geometry_point_from_utm(e, n, zone=None, ellps=None, srid=None):
             projections[srid] = p
         else:
             # p = pyproj.Proj(proj='utm', zone=int(zone), ellps='WGS84')
-            p = pyproj.Proj('EPSG:{}'.format(srid))
+            p = pyproj.Proj("EPSG:{}".format(srid))
 
     lon, lat = p(e, n, inverse=True)
     return make_geometry_point_from_latlon(lat, lon)
 
 
 def make_geometry_point_from_latlon(lat, lon):
-    return {'type': 'Point', 'coordinates': [lon, lat]}
+    return {"type": "Point", "coordinates": [lon, lat]}
 
 
 def iotid(iid):
-    return {'@iot.id': iid}
+    return {"@iot.id": iid}
 
 
 class STAClient:
@@ -87,10 +87,10 @@ class STAClient:
 
     @staticmethod
     def make_st_time(ts):
-        for fmt in ('%Y-%m-%d', '%Y-%m-%dT%H:%M:%S'):
+        for fmt in ("%Y-%m-%d", "%Y-%m-%dT%H:%M:%S"):
             try:
                 t = datetime.strptime(ts, fmt)
-                return f'{t.isoformat()}.000Z'
+                return f"{t.isoformat()}.000Z"
             except BaseException:
                 pass
         else:
@@ -98,135 +98,148 @@ class STAClient:
 
     def get_locations(self, fs=None, orderby=None):
         params = []
-        base = 'Locations'
+        base = "Locations"
         if fs:
-            params.append(f'$filter={fs}')
+            params.append(f"$filter={fs}")
         if orderby:
-            params.append(f'$orderby={orderby}')
+            params.append(f"$orderby={orderby}")
 
         if params:
-            params = '&'.join(params)
-            base = f'{base}?{params}'
+            params = "&".join(params)
+            base = f"{base}?{params}"
 
         url = self._make_url(base)
 
         return get_items(url)
 
     def delete_location(self, iotid):
-        url = self._make_url(f'Locations({iotid})')
+        url = self._make_url(f"Locations({iotid})")
         self.delete(url)
 
     def put_observed_property(self, name, description, **kw):
         obsprop_id = self.get_observed_property(name)
         if obsprop_id is None:
-            payload = {'name': name,
-                       'description': description,
-                       'definition': 'No Definition',
-                       }
-            obsprop_id = self._add('ObservedProperties', payload)
+            payload = {
+                "name": name,
+                "description": description,
+                "definition": "No Definition",
+            }
+            obsprop_id = self._add("ObservedProperties", payload)
 
         return obsprop_id
 
     def put_sensor(self, name, description):
         sensor_id = self.get_sensor(name)
         if sensor_id is None:
-            payload = {'name': name,
-                       'description': description,
-                       'encodingType': 'application/pdf',
-                       'metadata': 'No Metadata'
-                       }
-            sensor_id = self._add('Sensors', payload)
+            payload = {
+                "name": name,
+                "description": description,
+                "encodingType": "application/pdf",
+                "metadata": "No Metadata",
+            }
+            sensor_id = self._add("Sensors", payload)
 
         return sensor_id
 
-    def put_datastream(self, name, description, thing_id, obsprop_id, sensor_id,
-                       properties=None,
-                       unit=None, otype=None):
+    def put_datastream(
+        self,
+        name,
+        description,
+        thing_id,
+        obsprop_id,
+        sensor_id,
+        properties=None,
+        unit=None,
+        otype=None,
+    ):
 
         if unit is None:
             unit = FOOT
         if otype is None:
             otype = OM_Measurement
 
-        payload = {'Thing': iotid(thing_id),
-                   'ObservedProperty': iotid(obsprop_id),
-                   'Sensor': iotid(sensor_id),
-                   'unitOfMeasurement': unit,
-                   'observationType': otype,
-                   'description': description,
-                   'name': name}
+        payload = {
+            "Thing": iotid(thing_id),
+            "ObservedProperty": iotid(obsprop_id),
+            "Sensor": iotid(sensor_id),
+            "unitOfMeasurement": unit,
+            "observationType": otype,
+            "description": description,
+            "name": name,
+        }
 
         if properties:
-            payload['properties'] = properties
+            payload["properties"] = properties
 
         ds = self.get_datastream(name, thing_id)
         if ds:
-            ds_id = ds['@iot.id']
+            ds_id = ds["@iot.id"]
             if self._should_patch(ds, properties):
                 self.patch_datastream(ds_id, payload)
             added = False
         else:
-            ds_id = self._add('Datastreams', payload)
+            ds_id = self._add("Datastreams", payload)
             added = True
         return ds_id, added
 
     def get_sensor(self, name):
-        return self._get_id('Sensors', name)
+        return self._get_id("Sensors", name)
 
     def get_observed_property(self, name):
-        return self._get_id('ObservedProperties', name)
+        return self._get_id("ObservedProperties", name)
 
     def get_datastream(self, name, thing_id):
-        tag = f'Things({thing_id})/Datastreams'
+        tag = f"Things({thing_id})/Datastreams"
         vs = self._get_item_by_name(tag, name)
         if vs:
             return vs[0]
 
     def get_datastream_id(self, name, thing_id):
-        tag = f'Things({thing_id})/Datastreams'
+        tag = f"Things({thing_id})/Datastreams"
         return self._get_id(tag, name)
 
     def get_last_thing(self):
         pass
 
     def get_last_observation(self, datastream_id):
-        url = self._make_url(f'Datastreams({datastream_id})/Observations?$orderby=phenomenonTime desc&$top=1')
-        logging.info(f'request url: {url}')
+        url = self._make_url(
+            f"Datastreams({datastream_id})/Observations?$orderby=phenomenonTime desc&$top=1"
+        )
+        logging.info(f"request url: {url}")
         resp = requests.get(url)
         v = resp.json()
         # logging.info(f'v {v}')
 
-        vs = v.get('value')
+        vs = v.get("value")
         # logging.info(f'vs {vs}')
         if vs:
-            return vs[0].get('phenomenonTime')
+            return vs[0].get("phenomenonTime")
 
     def delete(self, url):
-        resp = requests.delete(url,
-                               auth=(self._user, self._pwd))
+        resp = requests.delete(url, auth=(self._user, self._pwd))
         if resp.status_code != 200:
             logging.info(resp, resp.text)
 
     def patch(self, url, payload):
-        resp = requests.patch(url,
-                              auth=(self._user, self._pwd),
-                              json=payload)
+        resp = requests.patch(url, auth=(self._user, self._pwd), json=payload)
         if resp.status_code != 200:
             logging.info(resp, resp.text)
 
     def patch_thing(self, iotid, payload):
-        url = self._make_url(f'Things({iotid})')
+        url = self._make_url(f"Things({iotid})")
         self.patch(url, payload)
 
     def patch_location(self, iotid, payload):
-        url = self._make_url(f'Locations({iotid})')
+        url = self._make_url(f"Locations({iotid})")
         self.patch(url, payload)
 
     def patch_datastream(self, iotid, payload):
-        url = self._make_url(f'Datastreams({iotid})')
+        url = self._make_url(f"Datastreams({iotid})")
         self.patch(url, payload)
 
-    def put_location(self, name, description, properties, utm=None, latlon=None, verbose=False):
+    def put_location(
+        self, name, description, properties, utm=None, latlon=None, verbose=False
+    ):
         lid = self.get_location_id(name)
         if lid is None:
 
@@ -237,33 +250,44 @@ class STAClient:
                 geometry = make_geometry_point_from_latlon(*latlon)
 
             if geometry:
-                payload = {'name': name,
-                           'description': description,
-                           'properties': properties,
-                           'location': geometry,
-                           'encodingType': 'application/vnd.geo+json'
-                           }
-                return self._add('Locations', payload, verbose=verbose), True
+                payload = {
+                    "name": name,
+                    "description": description,
+                    "properties": properties,
+                    "location": geometry,
+                    "encodingType": "application/vnd.geo+json",
+                }
+                return self._add("Locations", payload, verbose=verbose), True
             else:
-                logging.info('failed to construct geometry. need to specify utm or latlon')
+                logging.info(
+                    "failed to construct geometry. need to specify utm or latlon"
+                )
                 raise Exception
         else:
-            self.patch_location(lid, {'properties': properties, 'description': description})
+            self.patch_location(
+                lid, {"properties": properties, "description": description}
+            )
             return lid, False
 
-    def put_thing(self, name, description, properties, location_id, check=True, verbose=False):
+    def put_thing(
+        self, name, description, properties, location_id, check=True, verbose=False
+    ):
         tid = None
         if check:
             tid = self.get_thing_id(name, location_id)
 
         if tid is None:
-            payload = {'name': name,
-                       'description': description,
-                       'properties': properties,
-                       'Locations': [{'@iot.id': location_id}]}
-            return self._add('Things', payload, verbose=verbose)
+            payload = {
+                "name": name,
+                "description": description,
+                "properties": properties,
+                "Locations": [{"@iot.id": location_id}],
+            }
+            return self._add("Things", payload, verbose=verbose)
         else:
-            self.patch_thing(tid, {'properties': properties, 'description': description})
+            self.patch_thing(
+                tid, {"properties": properties, "description": description}
+            )
         return tid
 
     def add_observations(self, datastream_id, components, obs):
@@ -272,28 +296,28 @@ class STAClient:
 
         n = 100
         nobs = len(obs)
-        logging.info('nobservations: {}'.format(nobs))
+        logging.info("nobservations: {}".format(nobs))
         for i in range(0, nobs, n):
-            chunk = obs[i:i + n]
+            chunk = obs[i : i + n]
             pd = self.observation_payload(datastream_id, components, chunk)
             # logging.info('payload {}'.format(pd))
-            url = self._make_url('CreateObservations')
+            url = self._make_url("CreateObservations")
             # logging.info('url: {}'.format(url))
             # logging.info('payload: {}'.format(pd))
-            resp = requests.post(url,
-                                 auth=('write', self._pwd),
-                                 json=pd)
-            logging.info('response {}, {}'.format(i, resp))
+            resp = requests.post(url, auth=("write", self._pwd), json=pd)
+            logging.info("response {}, {}".format(i, resp))
 
     @staticmethod
     def observation_payload(datastream_id, components, data):
-        obj = {'Datastream': {'@iot.id': datastream_id},
-               'components': components,
-               'dataArray': data}
+        obj = {
+            "Datastream": {"@iot.id": datastream_id},
+            "components": components,
+            "dataArray": data,
+        }
         return [obj]
 
     def get_location_id(self, name):
-        return self._get_id('Locations', name)
+        return self._get_id("Locations", name)
 
     # def get_datastream_id(self, name, sensor_name, thing_id):
     #     tag = 'Datastreams'
@@ -303,14 +327,14 @@ class STAClient:
     #     return self._get_id(tag, name, extra_args=f'$filter=Sensor/name eq \'{sensor_name}\'')
 
     def get_thing(self, **filters):
-        base = self._make_base('Things', **filters)
+        base = self._make_base("Things", **filters)
         return self._get_item(base)
 
     def get_thing_id(self, name, location_id=None, location_name=None):
-        tag = 'Things'
-        extra_args=None
+        tag = "Things"
+        extra_args = None
         if location_id:
-            tag = f'Locations({location_id})/{tag}'
+            tag = f"Locations({location_id})/{tag}"
         elif location_name:
             extra_args = f"$filter=Location/name eq '{location_name}'"
 
@@ -319,90 +343,92 @@ class STAClient:
     @staticmethod
     def _should_patch(obj, properties):
         patch = True
-        if all((v == obj['properties'].get(k) for k, v in properties.items() if v)):
+        if all((v == obj["properties"].get(k) for k, v in properties.items() if v)):
             patch = False
-        elif all((v == properties.get(k) for k, v in obj['properties'].items())):
+        elif all((v == properties.get(k) for k, v in obj["properties"].items())):
             patch = False
         return patch
 
     @staticmethod
     def _make_base(tag, **filters):
         def factory(k, v):
-            k = k.replace('__', '/')
-            comp = 'eq'
-            return f'?filter={k} {comp} {v}'
+            k = k.replace("__", "/")
+            comp = "eq"
+            return f"?filter={k} {comp} {v}"
 
         fs = [factory(k, v) for k, v in filters.items()]
-        fs = '&'.join(fs)
-        return f'{tag}?{fs}'
+        fs = "&".join(fs)
+        return f"{tag}?{fs}"
 
     def _get_item(self, base, verbose=False):
         url = self._make_url(base)
-        resp = requests.get(url, auth=('read', 'read'))
+        resp = requests.get(url, auth=("read", "read"))
         if verbose:
-            logging.info(f'Get item {base}')
+            logging.info(f"Get item {base}")
 
         j = resp.json()
         try:
-            return j['value']
+            return j["value"]
         except KeyError:
             pass
 
     def _get_id(self, tag, name, verbose=False, **kw):
         vs = self._get_item_by_name(tag, name, **kw)
         if vs:
-            iotid = vs[0]['@iot.id']
+            iotid = vs[0]["@iot.id"]
             if verbose:
-                logging.info(f'Got tag={tag} name={name} iotid={iotid}')
+                logging.info(f"Got tag={tag} name={name} iotid={iotid}")
             return iotid
 
     def _get_item_by_name(self, tag, name, extra_args=None, verbose=False):
         tag = f"{tag}?$filter=name eq '{name}'"
         if extra_args:
-            tag = f'{tag}&{extra_args}'
+            tag = f"{tag}&{extra_args}"
         return self._get_item(tag, verbose)
 
     def _add(self, tag, payload, extract_iotid=True, verbose=False):
         url = self._make_url(tag)
         if verbose:
-            logging.info(f'Add url={url}')
-            logging.info(f'Add payload={payload}')
+            logging.info(f"Add url={url}")
+            logging.info(f"Add payload={payload}")
 
-        resp = requests.post(url,
-                             auth=(self._user, self._pwd),
-                             json=payload)
+        resp = requests.post(url, auth=(self._user, self._pwd), json=payload)
 
         if extract_iotid:
-            m = IDREGEX.search(resp.headers.get('location', ''))
+            m = IDREGEX.search(resp.headers.get("location", ""))
             # logging.info(f'Response={resp.json()}')
 
             if m:
-                iotid = m.group('id')[1:-1]
+                iotid = m.group("id")[1:-1]
                 if verbose:
-                    logging.info(f'added {tag} {iotid}')
+                    logging.info(f"added {tag} {iotid}")
                 return iotid
             else:
-                logging.info(f'failed adding {tag} {payload}')
-                logging.info(f'Response={resp.json()}')
+                logging.info(f"failed adding {tag} {payload}")
+                logging.info(f"Response={resp.json()}")
 
     def _make_url(self, tag):
         port = self._port
         if not port or port == 80:
-            port = ''
+            port = ""
         else:
-            port = f':{port}'
+            port = f":{port}"
 
-        return f'http://{self._host}{port}/FROST-Server/v1.1/{tag}'
+        return f"http://{self._host}{port}/FROST-Server/v1.1/{tag}"
 
 
 class STAMQTTClient:
     def __init__(self, host):
-        self._client = mqtt.Client('STA')
+        self._client = mqtt.Client("STA")
         self._client.connect(host)
 
     def add_observations(self, datastream_id, payloads):
         client = self._client
         for payload in payloads:
-            client.publish(f'v1.0/Datastreams({datastream_id})/Observations',
-                           payload=json.dumps(payload))
+            client.publish(
+                f"v1.0/Datastreams({datastream_id})/Observations",
+                payload=json.dumps(payload),
+            )
+
+
 # ============= EOF =============================================
